/* ---------------------------------------------------------------------------

  This file is part of NuRV (client sample code)
  Copyright (R) 2018-2021  Fondazione Bruno Kessler, Italy

  Author: Chun Tian <ctian@fbk.eu>
-----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <orbit/orbit.h>

#include "monitor.h"
#include "toolkit.h" /* ie. etk_abort_if_exception() */

static CORBA_ORB global_orb = CORBA_OBJECT_NIL; /* global orb */
static Monitor_MonitorService service = CORBA_OBJECT_NIL;
static CORBA_Environment ev[1];

/* Is called in case of process signals. it invokes CORBA_ORB_shutdown()
 * function, which will terminate the processes main loop.
 */
static void client_shutdown (int sig)
{
  CORBA_Environment  local_ev[1];
  CORBA_exception_init(local_ev);

  if (global_orb != CORBA_OBJECT_NIL) {
    CORBA_ORB_shutdown (global_orb, FALSE, local_ev);
    etk_abort_if_exception (local_ev, "caught exception");
  }
}

/* Inits ORB @orb using @argv arguments for configuration. For each
 * ORBit options consumed from vector @argv the counter of @argc_ptr
 * will be decremented. Signal handler is set to call
 * echo_client_shutdown function in case of SIGINT and SIGTERM
 * signals.  If error occures @ev points to exception object on
 * return.
 */
static void
client_init (int                    *argc_ptr,
	     char                   *argv[],
             CORBA_ORB              *orb,
             CORBA_Environment      *ev,
	     Monitor_MonitorService *service)
{
  CosNaming_NamingContext name_service = CORBA_OBJECT_NIL;
  gchar *id[] = {"NuRV", "Monitor", "Service", NULL};

  CORBA_exception_init(ev);
  /* init signal handling */
  signal(SIGINT,  client_shutdown);
  signal(SIGTERM, client_shutdown);

  /* create Object Request Broker (ORB) */
  (*orb) = CORBA_ORB_init(argc_ptr, argv, "orbit-local-orb", ev);
  if (etk_raised_exception(ev)) { return; }

  g_print ("Resolving service reference from name-service with id \"%s/%s/%s\"\n",
	   id[0], id[1], id[2]);

  name_service = etk_get_name_service (global_orb, ev);
  etk_abort_if_exception(ev, "failed resolving name-service");

  *service = (Monitor_MonitorService) etk_name_service_resolve (name_service, id, ev);
  etk_abort_if_exception(ev, "failed resolving service at name-service");

}

/* Releases @servant object and finally destroys @orb. If error
 * occures @ev points to exception object on return.
 */
static void client_cleanup (CORBA_ORB          orb,
			    CORBA_Object       service,
			    CORBA_Environment *ev)
{
  /* releasing managed object */
  CORBA_Object_release(service, ev);
  if (etk_raised_exception(ev)) return;

  /* tear down the ORB */
  if (orb != CORBA_OBJECT_NIL) {
    /* going to destroy orb.. */
    CORBA_ORB_destroy(orb, ev);
    if (etk_raised_exception(ev)) return;
  }
}

void print_output(Monitor_Verdict res)
{
  switch (res) {
  case Monitor_RV_True:
    g_print("true\n");
    break;
  case Monitor_RV_False:
    g_print("false\n");
    break;
  case Monitor_RV_Unknown:
    g_print("unknown\n");
    break;
  default:
    g_print("error\n");
  }
}

/*
 * main function
 */
int main (int argc, char* argv[])
{
  Monitor_Verdict res;

  /* Startup messages */
  g_print("*** Usage: %s -ORBInitRef NameService=IOR:... \n\n", argv[0]);

  client_init (&argc, argv, &global_orb, ev, &service);
  etk_abort_if_exception(ev, "init failed");

  CORBA_long id = 0;
  CORBA_any index;
  index._type = TC_CORBA_long;
  index._value = &id;

  CORBA_char *state_p = "p";
  CORBA_char *state_q = "q";

  /* now send p,p,p,q,q,q to the monitor and print the outputs */
  res = Monitor_MonitorService_heartbeat (service, &index, state_p, ev);
  print_output(res);
  res = Monitor_MonitorService_heartbeat (service, &index, state_p, ev);
  print_output(res);
  res = Monitor_MonitorService_heartbeat (service, &index, state_p, ev);
  print_output(res);
  res = Monitor_MonitorService_heartbeat (service, &index, state_q, ev);
  print_output(res);
  res = Monitor_MonitorService_heartbeat (service, &index, state_q, ev);
  print_output(res);
  res = Monitor_MonitorService_heartbeat (service, &index, state_q, ev);
  print_output(res);

  /* demo: how to reset the monitor */
  CORBA_boolean hard_p = CORBA_TRUE;
  Monitor_MonitorService_reset (service, &index, hard_p, ev);

  client_cleanup (global_orb, service, ev);
  etk_abort_if_exception(ev, "cleanup failed");

  exit (0);
}
