// ---------------------------------------------------------------------------
//
//  This file is part of NuRV (client sample code)
//  Copyright (R) 2018-2021  Fondazione Bruno Kessler, Italy
//
//  Author: Chun Tian <ctian@fbk.eu>
// ---------------------------------------------------------------------------

#include <iostream>
#include "monitor.hh"

using namespace std;

static CORBA::Object_ptr getObjectReference(CORBA::ORB_ptr orb)
{
  CosNaming::NamingContext_var rootContext;

  try {
    // Obtain a reference to the root context of the Name service:
    CORBA::Object_var obj;
    obj = orb->resolve_initial_references("NameService");

    // Narrow the reference returned.
    rootContext = CosNaming::NamingContext::_narrow(obj);
    if (CORBA::is_nil(rootContext)) {
      cerr << "Failed to narrow the root naming context." << endl;
      return 0;
    }
  }
  catch (CORBA::NO_RESOURCES&) {
    cerr << "Caught NO_RESOURCES exception. You must configure omniORB "
	 << "with the location" << endl
	 << "of the naming service. (-ORBInitRef NameService=IOR:...)"
	 << endl;
    return 0;
  }
  catch (CORBA::ORB::InvalidName&) {
    // This should not happen!
    cerr << "Service required is invalid [does not exist]." << endl;
    return 0;
  }

  // Create a name object, containing the name FBK/Monitor/Service:
  CosNaming::Name name;
  name.length(3);
  name[0].id = (const char*) "NuRV"; // string copied
  name[0].kind = (const char*) "";  // string copied
  name[1].id = (const char*) "Monitor";
  name[1].kind = (const char*) "";
  name[2].id = (const char*) "Service";
  name[2].kind = (const char*) "";
  // Note on kind: The kind field is used to indicate the type
  // of the object. This is to avoid conventions such as that used
  // by files (name.type -- e.g. test.ps = postscript etc.)

  try {
    // Resolve the name to an object reference.
    return rootContext->resolve(name);
  }
  catch (CosNaming::NamingContext::NotFound& ex) {
    // This exception is thrown if any of the components of the
    // path [contexts or the object] aren’t found:
    cerr << "Context not found." << endl;
  }
  catch (CORBA::TRANSIENT& ex) {
    cerr << "Caught system exception TRANSIENT -- unable to contact the "
	 << "naming service." << endl
	 << "Make sure the naming server is running and that omniORB is "
	 << "configured correctly (-ORBInitRef NameService=IOR:...)"
	 << endl;
  }
  catch (CORBA::SystemException& ex) {
    cerr << "Caught a CORBA::" << ex._name()
	 << " while using the naming service." << endl;
  }

  return CORBA::Object::_nil();
}

int main(int argc, char **argv)
{
  try {
    // Initialise the ORB.
    CORBA::ORB_ptr orb = CORBA::ORB_init(argc, argv, "omniORB4");
   
    CORBA::Object_var obj = getObjectReference(orb);
    Monitor::MonitorService_var service = Monitor::MonitorService::_narrow(obj);
    if (CORBA::is_nil(service)) {
      cerr << "cannot invoke on a nil object reference." << endl;
      return -1;
    }

    // the input state
    CORBA::String_var state = (const char*) "TRUE";
    Monitor::Verdict res;

    CORBA::Any index;
    CORBA::Long l = 0;
    index <<= l;

    res = service->heartbeat (index, state);    
    string result;

    switch (res) {
    case Monitor::RV_True:
      result = "true";
      break;
    case Monitor::RV_False:
      result = "false";
      break;
    case Monitor::RV_Unknown:
      result = "unknown";
      break;
    default:
      result = "error";
    }

    cout << "result = " << result << endl;

    // reset the monitor
    CORBA::Boolean hard_p = false;
    service->reset (index, hard_p);

    orb->destroy();
  }
  catch (CORBA::TRANSIENT&) {
    cerr << "Caught system exception TRANSIENT -- unable to contact the "
	 << "server." << endl;
  }
  catch (CORBA::SystemException& ex) {
    cerr << "Caught a CORBA::" << ex._name() << endl;
  }
  catch (CORBA::Exception& ex) {
    cerr << "Caught CORBA::Exception: " << ex._name() << endl;
  }
  return 0;
}
